from setuptools import setup

setup(
    name='XBDistroTools',
    version='0.1.0',
    packages=['XBStrapDistro', 'XBStrapSQLite'],
    url='https://electrode.codes',
    author='electrodeyt',
    author_email='electrodeyt@gmail.com',
    description='A set of tools to interface with xbstrap distributions.'
)
